@@ -1,17 +1,20 @@
 import time
+import traceback
 import unittest
 import xml.dom.minidom
-import traceback
 
 import stomp
+from stomp.exception import *
 from stomp.listener import *
-
 from stomp.test.testutils import *
 
 
-class TransformationListener(WaitingListener):
+log = logging.getLogger('stomp.py')
+
+
+class TransformationListener(TestListener):
     def __init__(self, receipt):
-        WaitingListener.__init__(self, receipt)
+        TestListener.__init__(self, receipt)
         self.message = None
 
     def on_before_message(self, headers, body):
@@ -27,7 +30,8 @@ class TransformationListener(WaitingListener):
                 for entryElem in rootElem.getElementsByTagName("entry"):
                     pair = []
                     for node in entryElem.childNodes:
-                        if not isinstance(node, xml.dom.minidom.Element): continue
+                        if not isinstance(node, xml.dom.minidom.Element):
+                            continue
                         pair.append(node.firstChild.nodeValue)
                     assert len(pair) == 2
                     entries[pair[0]] = pair[1]
@@ -40,17 +44,17 @@ class TransformationListener(WaitingListener):
                 return (headers, body)
 
     def on_message(self, headers, body):
+        TestListener.on_message(self, headers, body)
         self.message = body
 
 
 class TestMessageTransform(unittest.TestCase):
 
     def setUp(self):
-        conn = stomp.Connection(get_standard_host())
+        conn = stomp.Connection(get_rabbitmq_host())
         listener = TransformationListener('123')
         conn.set_listener('', listener)
-        conn.start()
-        conn.connect('admin', 'password', wait=True)
+        conn.connect(get_rabbitmq_user(), get_rabbitmq_password(), wait=True)
         self.conn = conn
         self.listener = listener
         self.timestamp = time.strftime('%Y%m%d%H%M%S')
@@ -59,7 +63,7 @@ class TestMessageTransform(unittest.TestCase):
         if self.conn:
             self.conn.disconnect(receipt=None)
 
-    def testTransform(self):
+    def test_transform(self):
         queuename = '/queue/testtransform-%s' % self.timestamp
         self.conn.subscribe(destination=queuename, id=1, ack='auto')
 
@@ -72,11 +76,40 @@ class TestMessageTransform(unittest.TestCase):
         <string>city</string>
         <string>Belgrade</string>
     </entry>
-</map>''', destination=queuename, headers={'transformation':'jms-map-xml'}, receipt='123')
+</map>''', destination=queuename, headers={'transformation': 'jms-map-xml'}, receipt='123')
 
         self.listener.wait_on_receipt()
+        self.listener.wait_for_message()
+
+        self.assertTrue(self.listener.message is not None, 'Did not receive a message')
+        self.assertTrue(self.listener.message.__class__ == dict,
+            'Message type should be dict after transformation, was %s' % self.listener.message.__class__)
+        self.assertTrue(self.listener.message['name'] == 'Dejan', 'Missing an expected dict element')
+        self.assertTrue(self.listener.message['city'] == 'Belgrade', 'Missing an expected dict element')
+
+
+class TestNoResponseConnectionKill(unittest.TestCase):
+    def setUp(self):
+        self.server = TestStompServer('127.0.0.1', 60000)
+        self.server.start()
+        self.timeout_thread = threading.Thread(name='shutdown test server', target=self.timeout_server)
+
+    def timeout_server(self):
+        time.sleep(3)
+        log.info('Stopping server')
+        self.server.running = False
+        self.server.stop()
+
+    def test_noresponse(self):
+        try:
+            conn = stomp.Connection([('127.0.0.1', 60000)], heartbeats=(1000, 1000))
+            listener = TestListener()
+            conn.set_listener('', listener)
+            self.timeout_thread.start()
+            conn.connect(wait=True)
+            self.fail("Shouldn't happen")
+        except ConnectFailedException:
+            log.info('Received connect failed - test success')
+        except Exception:
+            self.fail("Shouldn't happen")
 
-        self.assert_(self.listener.message is not None, 'Did not receive a message')
-        self.assert_(self.listener.message.__class__ == dict, 'Message type should be dict after transformation, was %s' % self.listener.message.__class__)
-        self.assert_(self.listener.message['name'] == 'Dejan', 'Missing an expected dict element')
-        self.assert_(self.listener.message['city'] == 'Belgrade', 'Missing an expected dict element')
