@@ -1,10 +1,11 @@
 import unittest
 
 import stomp
-
+from stomp.listener import TestListener
 from stomp.test.testutils import *
 
-class TestRabbitMQSend(unittest.TestCase):
+
+class TestStompServerSend(unittest.TestCase):
 
     def setUp(self):
         pass
@@ -13,18 +14,18 @@ class TestRabbitMQSend(unittest.TestCase):
         conn = stomp.Connection10(get_stompserver_host())
         listener = TestListener('123')
         conn.set_listener('', listener)
-        conn.start()
         conn.connect(wait=True)
         conn.subscribe(destination='/queue/test', ack='auto')
 
         conn.send(body='this is a test', destination='/queue/test', receipt='123')
 
         listener.wait_on_receipt()
+        listener.wait_for_message()
 
         conn.unsubscribe('/queue/test')
 
         conn.disconnect(receipt=None)
 
-        self.assert_(listener.connections == 1, 'should have received 1 connection acknowledgement')
-        self.assert_(listener.messages == 1, 'should have received 1 message')
-        self.assert_(listener.errors == 0, 'should not have received any errors')
+        self.assertTrue(listener.connections == 1, 'should have received 1 connection acknowledgement')
+        self.assertTrue(listener.messages == 1, 'should have received 1 message')
+        self.assertTrue(listener.errors == 0, 'should not have received any errors')
