@@ -1,22 +1,29 @@
+try:
+    from configparser import RawConfigParser
+except ImportError:
+    from ConfigParser import RawConfigParser
+import logging
 import os
+import re
 import socket
 import threading
-import logging
-log = logging.getLogger('testutils.py')
 
-try:
-    from configparser import ConfigParser
-except ImportError:
-    from ConfigParser import ConfigParser
-
-
-from stomp import StatsListener, WaitingListener
 from stomp.backward import *
 
 
-config = ConfigParser()
+log = logging.getLogger('testutils.py')
+
+config = RawConfigParser()
 config.read(os.path.join(os.path.dirname(__file__), 'setup.ini'))
 
+header_re = re.compile(r'[^:]+:.*')
+
+
+if sys.hexversion >= 0x03000000:  # Python 3+
+    from stomp.test.p3testutils import *
+else:  # Python 2
+    from stomp.test.p2testutils import *
+
 
 def get_environ(name):
     try:
@@ -24,56 +31,74 @@ def get_environ(name):
     except:
         return None
 
-def get_standard_host():
+
+def get_default_host():
     host = config.get('default', 'host')
     port = config.get('default', 'port')
     return [(get_environ('STD_HOST') or host, int(get_environ('STD_PORT') or port))]
 
 
-def get_standard_ssl_host():
+def get_default_vhost():
+    try:
+        vhost = config.get('default', 'vhost')
+    except:
+        vhost = None
+    return get_environ('STD_VHOST') or vhost
+
+
+def get_default_user():
+    user = config.get('default', 'user')
+    return get_environ('STD_USER') or user
+
+
+def get_default_password():
+    password = config.get('default', 'password')
+    return get_environ('STD_PASSWORD') or password
+
+
+def get_ipv6_host():
+    host = config.get('ipv6', 'host')
+    port = config.get('ipv6', 'port')
+    return [(get_environ('IPV6_HOST') or host, int(get_environ('IPV6_PORT') or port))]
+
+
+def get_default_ssl_host():
     host = config.get('default', 'host')
     port = config.get('default', 'ssl_port')
     return [(get_environ('STD_HOST') or host, int(get_environ('STD_SSL_PORT') or port))]
 
 
+def get_sni_ssl_host():
+    host = config.get('sni', 'host')
+    port = config.get('sni', 'ssl_port')
+    return [(get_environ('SNI_HOST') or host, int(get_environ('SNI_SSL_PORT') or port))]
+
+
 def get_rabbitmq_host():
     host = config.get('rabbitmq', 'host')
     port = config.get('rabbitmq', 'port')
     return [(get_environ('RABBITMQ_HOST') or host, int(get_environ('RABBITMQ_PORT') or port))]
 
 
+def get_rabbitmq_user():
+    user = config.get('rabbitmq', 'user')
+    return get_environ('RABBITMQ_USER') or user
+
+
+def get_rabbitmq_password():
+    password = config.get('rabbitmq', 'password')
+    return get_environ('RABBITMQ_PASSWORD') or password
+
+
 def get_stompserver_host():
     host = config.get('stompserver', 'host')
     port = config.get('stompserver', 'port')
     return [(get_environ('STOMPSERVER_HOST') or host, int(get_environ('STOMPSERVER_PORT') or port))]
 
-
-class TestListener(StatsListener,WaitingListener):
-    def __init__(self, receipt=None):
-        StatsListener.__init__(self)
-        WaitingListener.__init__(self, receipt)
-        self.message_list = []
-        self.message_condition = threading.Condition()
-        self.message_received = False
-
-    def on_message(self, headers, message):
-        StatsListener.on_message(self, headers, message)
-        self.message_list.append((headers, message))
-        self.message_condition.acquire()
-        self.message_received = True
-        self.message_condition.notify()
-        self.message_condition.release()
-
-    def wait_for_message(self):
-        self.message_condition.acquire()
-        while not self.message_received:
-            self.message_condition.wait()
-        self.message_condition.release()
-        self.message_received = False
-
-    def get_latest_message(self):
-        return self.message_list[len(self.message_list)-1]
-
+def get_apollo_host():
+    host = config.get('apollo', 'host')
+    port = config.get('apollo', 'port')
+    return [(get_environ('APOLLO_HOST') or host, int(get_environ('APOLLO_PORT') or port))]
 
 class TestStompServer(object):
     def __init__(self, host, port):
@@ -122,10 +147,10 @@ class TestStompServer(object):
         self.frames.append(frame)
 
     def run(self):
-        self.conn, addr = self.s.accept()
+        self.conn, _ = self.s.accept()
         while self.running:
             try:
-                data = self.conn.recv(1024)
+                _ = self.conn.recv(1024)
                 frame = self.get_next_frame()
                 if self.conn is None:
                     break
@@ -141,3 +166,37 @@ class TestStompServer(object):
             pass
         self.stopped = True
         log.debug('Run loop completed')
+
+
+class TestStdin(object):
+    pass
+
+
+class TestStdout(object):
+    def __init__(self, test):
+        self.test = test
+        self.expects = []
+
+    def expect(self, txt):
+        self.expects.insert(0, re.compile(txt))
+
+    def write(self, txt):
+        txt = txt.rstrip()
+        if txt != '':
+            print(txt)
+        if txt == '>' or txt == '' or header_re.match(txt):
+            return
+        if len(self.expects) == 0:
+            self.test.fail('No expectations - actual "%s"' % txt)
+            return
+
+        for x in range(0, len(self.expects)):
+            chk = self.expects[x]
+            if chk.match(txt):
+                del self.expects[x]
+                return
+
+        self.test.fail('"%s" was not expected' % txt)
+
+    def flush(self):
+        pass
